extends KinematicBody2D

export(int) var speed = 25
export(float) var flank_distance = 50
var ball = null

var temp = 0

var offpoint
var defpoint
var midpoint

## MOVEMENT
func control_player(vel):#Moves this character around
	if(collision_layer > 0):#Can't walk if tripped
		if(vel.x < 0):#face left
			$Sprite.scale.x = -1
		elif(vel.x > 0):#face right
			$Sprite.scale.x = 1
		else:
			$AnimationPlayer.stop(true)
		if(vel.x + vel.y != 0):#The character is moving
			if($AnimationPlayer.is_playing() == false):#Don't stop the dance
				$AnimationPlayer.play("walk")
		var colid = move_and_collide((vel*speed)/56)
		if(colid != null):
			if(colid.collider.get_class() == "KinematicBody2D"):
				if($AnimationPlayer.current_animation == "tripper"):
					colid.collider.tripped()

func tempboost(val,time):#Temporary speed boost
	temp = val
	speed += temp
	var clock = Timer.new()
	clock.start(time)
	clock.connect("timeout",self,"boostend",[clock])
	add_child(clock)
	get_node("CPUParticles2D").emitting = true
	$AnimationPlayer.play("samba")

func boostend(clock):#The temporary speed boost has ended
	speed -= temp
	temp = 0
	clock.queue_free()
	get_node("CPUParticles2D").emitting = false
	$AnimationPlayer.play("walk")

## TRIPPING
func tripper():#This character tries to stun lock a character
	$AnimationPlayer.play("tripper")
	
	if(is_network_master()):
		rpc("update_tripper")#if in a online game

func tripped():#This character is stun locked for a second
	$AnimationPlayer.play("tripped")
	$AnimationPlayer.connect("animation_finished",self,"get_up")
	modulate = Color(0.4,0.4,0.4,0.7)
	collision_layer = 0
	collision_mask = 0

	if(get_tree().network_peer != null):
		if(is_network_master()):
			rpc("update_tripped")#if in a online game

func get_up(anime):#This character is done being stun locked
	$AnimationPlayer.disconnect("animation_finished",self,"get_up")
	modulate = Color(1,1,1)
	collision_layer = 1
	collision_mask = 1

	if(get_tree().network_peer != null):
		if(is_network_master()):
			rpc("update_getup")#if in a online game

## AUTO AI
#wip!
func chase():#targets the tail of the ball (ie the inverse direction the ball is going)
	var ball = get_node("../../../ball")
	if(is_instance_valid(ball)):
		var tail = ball.global_position - ball.linear_velocity
		var direction = tail - global_position
		direction.x = clamp(round(direction.x*4),-1,1)
		direction.y = clamp(round(direction.y*4),-1,1)
		control_player(direction)
	else:
		print_debug("ERROR could not get ball")

func flank():#Targets the side of the soccor ball
	var ball = get_node("../../../ball")
	if(is_instance_valid(ball)):
		var tail = ball.position - ball.linear_velocity
		var approach = ball.linear_velocity.rotated(ball.rotation)
		var direction = (approach + ball.global_position) - global_position
		direction.x = clamp(direction.x,-1,1)
		direction.y = clamp(direction.y,-1,1)
		control_player(direction)
	else:
		print_debug("ERROR could not get ball")

func goto_point(point):#Heads back to it's area of the game
	if(is_instance_valid(point)):
		var direction = point.global_position - global_position
#		print_debug(direction, point.global_position,global_position)
		direction.x = clamp(direction.x,-1,1)
		direction.y = clamp(direction.y,-1,1)
		control_player(direction)
#	else:
#		print_debug(point)

## ONLINE
slave func update_pos(pos):
	position = pos

slave func update_tripped():
	tripped()

slave func update_tripper():
	tripper()

slave func update_getup():
	get_up("")

## MAIN
master func _physics_process(delta):
	if(is_instance_valid(ball) == false):
		ball = get_node("../../../ball")
	if(get_node("../..").select_character != self):
		match(get_node("../..").stance):
			1:#MIDDLE
				if(ball.position.distance_to(midpoint.position) < flank_distance):
					flank()
				else:
					goto_point(midpoint)
			2:#OFFENSIVE
				if(ball.position.distance_to(offpoint.position) < flank_distance):
					flank()
				else:
					goto_point(offpoint)
			0:#DEFENSIVE
				if(ball.position.distance_to(defpoint.position) < flank_distance):
					flank()
				else:
					goto_point(defpoint)
	
	if(get_tree().network_peer != null):
		if(is_network_master()):
			rpc("update_pos",position)

func _ready():
	midpoint = get_node("../../Navigation2D/"+name+"/middle")
	defpoint = get_node("../../Navigation2D/"+name+"/defensive")
	offpoint = get_node("../../Navigation2D/"+name+"/offensive")
	global_position = midpoint.global_position